# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .self_service_profile_sso_ticket_idp_initiated_options import SelfServiceProfileSsoTicketIdpInitiatedOptions


class SelfServiceProfileSsoTicketConnectionOptions(UniversalBaseModel):
    """
    The connection's options (depend on the connection strategy)
    """

    icon_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL for the icon. Must use HTTPS.
    """

    domain_aliases: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    List of domain_aliases that can be authenticated in the Identity Provider
    """

    idpinitiated: typing.Optional[SelfServiceProfileSsoTicketIdpInitiatedOptions] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class GetGroupResponseContent(UniversalBaseModel):
    """
    Represents the metadata of a group. Member lists are retrieved via a separate endpoint.
    """

    id: str = pydantic.Field()
    """
    Unique identifier for the group (service-generated).
    """

    name: str = pydantic.Field()
    """
    Name of the group. Must be unique within its connection. Must contain between 1 and 128 printable ASCII characters.
    """

    external_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    External identifier for the group, often used for SCIM synchronization. Max length of 256 characters.
    """

    connection_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Identifier for the connection this group belongs to (if a connection group).
    """

    tenant_name: str = pydantic.Field()
    """
    Identifier for the tenant this group belongs to.
    """

    created_at: dt.datetime = pydantic.Field()
    """
    Timestamp of when the group was created.
    """

    updated_at: dt.datetime = pydantic.Field()
    """
    Timestamp of when the group was last updated.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .client_addon_saml_mapping import ClientAddonSamlMapping


class ClientAddonSaml(UniversalBaseModel):
    """
    SAML2 addon indicator (no configuration settings needed for SAML2 addon).
    """

    mappings: typing.Optional[ClientAddonSamlMapping] = None
    audience: typing.Optional[str] = None
    recipient: typing.Optional[str] = None
    create_upn_claim: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="createUpnClaim"), pydantic.Field(alias="createUpnClaim")
    ] = None
    map_unknown_claims_as_is: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="mapUnknownClaimsAsIs"), pydantic.Field(alias="mapUnknownClaimsAsIs")
    ] = None
    passthrough_claims_with_no_mapping: typing_extensions.Annotated[
        typing.Optional[bool],
        FieldMetadata(alias="passthroughClaimsWithNoMapping"),
        pydantic.Field(alias="passthroughClaimsWithNoMapping"),
    ] = None
    map_identities: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="mapIdentities"), pydantic.Field(alias="mapIdentities")
    ] = None
    signature_algorithm: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="signatureAlgorithm"), pydantic.Field(alias="signatureAlgorithm")
    ] = None
    digest_algorithm: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="digestAlgorithm"), pydantic.Field(alias="digestAlgorithm")
    ] = None
    issuer: typing.Optional[str] = None
    destination: typing.Optional[str] = None
    lifetime_in_seconds: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="lifetimeInSeconds"), pydantic.Field(alias="lifetimeInSeconds")
    ] = None
    sign_response: typing_extensions.Annotated[
        typing.Optional[bool], FieldMetadata(alias="signResponse"), pydantic.Field(alias="signResponse")
    ] = None
    name_identifier_format: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="nameIdentifierFormat"), pydantic.Field(alias="nameIdentifierFormat")
    ] = None
    name_identifier_probes: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="nameIdentifierProbes"),
        pydantic.Field(alias="nameIdentifierProbes"),
    ] = None
    authn_context_class_ref: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="authnContextClassRef"), pydantic.Field(alias="authnContextClassRef")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_xml_parse_xml import FlowActionXmlParseXml
from .flow_action_xml_serialize_xml import FlowActionXmlSerializeXml

FlowActionXml = typing.Union[FlowActionXmlParseXml, FlowActionXmlSerializeXml]



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionAcrValuesSupported = typing.List[str]
"""
A list of the Authentication Context Class References that this OP supports
"""



from .conftest import get_client, verify_request_count


def test_connectionProfiles_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "connection_profiles.list_.0"
    client = get_client(test_id)
    client.connection_profiles.list(
        from_="from",
        take=1,
    )
    verify_request_count(test_id, "GET", "/connection-profiles", {"from": "from", "take": "1"}, 1)


def test_connectionProfiles_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "connection_profiles.create.0"
    client = get_client(test_id)
    client.connection_profiles.create(
        name="name",
    )
    verify_request_count(test_id, "POST", "/connection-profiles", None, 1)


def test_connectionProfiles_list_templates() -> None:
    """Test listTemplates endpoint with WireMock"""
    test_id = "connection_profiles.list_templates.0"
    client = get_client(test_id)
    client.connection_profiles.list_templates()
    verify_request_count(test_id, "GET", "/connection-profiles/templates", None, 1)


def test_connectionProfiles_get_template() -> None:
    """Test getTemplate endpoint with WireMock"""
    test_id = "connection_profiles.get_template.0"
    client = get_client(test_id)
    client.connection_profiles.get_template(
        id="id",
    )
    verify_request_count(test_id, "GET", "/connection-profiles/templates/id", None, 1)


def test_connectionProfiles_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "connection_profiles.get.0"
    client = get_client(test_id)
    client.connection_profiles.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/connection-profiles/id", None, 1)


def test_connectionProfiles_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "connection_profiles.delete.0"
    client = get_client(test_id)
    client.connection_profiles.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/connection-profiles/id", None, 1)


def test_connectionProfiles_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "connection_profiles.update.0"
    client = get_client(test_id)
    client.connection_profiles.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/connection-profiles/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import encode_path_param
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.bad_request_error import BadRequestError
from ...errors.conflict_error import ConflictError
from ...errors.forbidden_error import ForbiddenError
from ...errors.not_found_error import NotFoundError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.branding_theme_borders import BrandingThemeBorders
from ...types.branding_theme_colors import BrandingThemeColors
from ...types.branding_theme_fonts import BrandingThemeFonts
from ...types.branding_theme_identifiers import BrandingThemeIdentifiers
from ...types.branding_theme_page_background import BrandingThemePageBackground
from ...types.branding_theme_widget import BrandingThemeWidget
from ...types.create_branding_theme_response_content import CreateBrandingThemeResponseContent
from ...types.get_branding_default_theme_response_content import GetBrandingDefaultThemeResponseContent
from ...types.get_branding_theme_response_content import GetBrandingThemeResponseContent
from ...types.update_branding_theme_response_content import UpdateBrandingThemeResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawThemesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        borders: BrandingThemeBorders,
        colors: BrandingThemeColors,
        fonts: BrandingThemeFonts,
        page_background: BrandingThemePageBackground,
        widget: BrandingThemeWidget,
        display_name: typing.Optional[str] = OMIT,
        identifiers: typing.Optional[BrandingThemeIdentifiers] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateBrandingThemeResponseContent]:
        """
        Create branding theme.

        Parameters
        ----------
        borders : BrandingThemeBorders

        colors : BrandingThemeColors

        fonts : BrandingThemeFonts

        page_background : BrandingThemePageBackground

        widget : BrandingThemeWidget

        display_name : typing.Optional[str]
            Display Name

        identifiers : typing.Optional[BrandingThemeIdentifiers]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateBrandingThemeResponseContent]
            Branding settings successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "branding/themes",
            method="POST",
            json={
                "borders": convert_and_respect_annotation_metadata(
                    object_=borders, annotation=BrandingThemeBorders, direction="write"
                ),
                "colors": convert_and_respect_annotation_metadata(
                    object_=colors, annotation=BrandingThemeColors, direction="write"
                ),
                "displayName": display_name,
                "fonts": convert_and_respect_annotation_metadata(
                    object_=fonts, annotation=BrandingThemeFonts, direction="write"
                ),
                "identifiers": convert_and_respect_annotation_metadata(
                    object_=identifiers, annotation=BrandingThemeIdentifiers, direction="write"
                ),
                "page_background": convert_and_respect_annotation_metadata(
                    object_=page_background, annotation=BrandingThemePageBackground, direction="write"
                ),
                "widget": convert_and_respect_annotation_metadata(
                    object_=widget, annotation=BrandingThemeWidget, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateBrandingThemeResponseContent,
                    parse_obj_as(
                        type_=CreateBrandingThemeResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_default(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetBrandingDefaultThemeResponseContent]:
        """
        Retrieve default branding theme.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetBrandingDefaultThemeResponseContent]
            Branding theme successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "branding/themes/default",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBrandingDefaultThemeResponseContent,
                    parse_obj_as(
                        type_=GetBrandingDefaultThemeResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, theme_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetBrandingThemeResponseContent]:
        """
        Retrieve branding theme.

        Parameters
        ----------
        theme_id : str
            The ID of the theme

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetBrandingThemeResponseContent]
            Branding theme successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"branding/themes/{encode_path_param(theme_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBrandingThemeResponseContent,
                    parse_obj_as(
                        type_=GetBrandingThemeResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, theme_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Delete branding theme.

        Parameters
        ----------
        theme_id : str
            The ID of the theme

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"branding/themes/{encode_path_param(theme_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        theme_id: str,
        *,
        borders: BrandingThemeBorders,
        colors: BrandingThemeColors,
        fonts: BrandingThemeFonts,
        page_background: BrandingThemePageBackground,
        widget: BrandingThemeWidget,
        display_name: typing.Optional[str] = OMIT,
        identifiers: typing.Optional[BrandingThemeIdentifiers] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateBrandingThemeResponseContent]:
        """
        Update branding theme.

        Parameters
        ----------
        theme_id : str
            The ID of the theme

        borders : BrandingThemeBorders

        colors : BrandingThemeColors

        fonts : BrandingThemeFonts

        page_background : BrandingThemePageBackground

        widget : BrandingThemeWidget

        display_name : typing.Optional[str]
            Display Name

        identifiers : typing.Optional[BrandingThemeIdentifiers]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateBrandingThemeResponseContent]
            Branding settings successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"branding/themes/{encode_path_param(theme_id)}",
            method="PATCH",
            json={
                "borders": convert_and_respect_annotation_metadata(
                    object_=borders, annotation=BrandingThemeBorders, direction="write"
                ),
                "colors": convert_and_respect_annotation_metadata(
                    object_=colors, annotation=BrandingThemeColors, direction="write"
                ),
                "displayName": display_name,
                "fonts": convert_and_respect_annotation_metadata(
                    object_=fonts, annotation=BrandingThemeFonts, direction="write"
                ),
                "identifiers": convert_and_respect_annotation_metadata(
                    object_=identifiers, annotation=BrandingThemeIdentifiers, direction="write"
                ),
                "page_background": convert_and_respect_annotation_metadata(
                    object_=page_background, annotation=BrandingThemePageBackground, direction="write"
                ),
                "widget": convert_and_respect_annotation_metadata(
                    object_=widget, annotation=BrandingThemeWidget, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBrandingThemeResponseContent,
                    parse_obj_as(
                        type_=UpdateBrandingThemeResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawThemesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        borders: BrandingThemeBorders,
        colors: BrandingThemeColors,
        fonts: BrandingThemeFonts,
        page_background: BrandingThemePageBackground,
        widget: BrandingThemeWidget,
        display_name: typing.Optional[str] = OMIT,
        identifiers: typing.Optional[BrandingThemeIdentifiers] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateBrandingThemeResponseContent]:
        """
        Create branding theme.

        Parameters
        ----------
        borders : BrandingThemeBorders

        colors : BrandingThemeColors

        fonts : BrandingThemeFonts

        page_background : BrandingThemePageBackground

        widget : BrandingThemeWidget

        display_name : typing.Optional[str]
            Display Name

        identifiers : typing.Optional[BrandingThemeIdentifiers]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateBrandingThemeResponseContent]
            Branding settings successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "branding/themes",
            method="POST",
            json={
                "borders": convert_and_respect_annotation_metadata(
                    object_=borders, annotation=BrandingThemeBorders, direction="write"
                ),
                "colors": convert_and_respect_annotation_metadata(
                    object_=colors, annotation=BrandingThemeColors, direction="write"
                ),
                "displayName": display_name,
                "fonts": convert_and_respect_annotation_metadata(
                    object_=fonts, annotation=BrandingThemeFonts, direction="write"
                ),
                "identifiers": convert_and_respect_annotation_metadata(
                    object_=identifiers, annotation=BrandingThemeIdentifiers, direction="write"
                ),
                "page_background": convert_and_respect_annotation_metadata(
                    object_=page_background, annotation=BrandingThemePageBackground, direction="write"
                ),
                "widget": convert_and_respect_annotation_metadata(
                    object_=widget, annotation=BrandingThemeWidget, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateBrandingThemeResponseContent,
                    parse_obj_as(
                        type_=CreateBrandingThemeResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_default(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetBrandingDefaultThemeResponseContent]:
        """
        Retrieve default branding theme.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetBrandingDefaultThemeResponseContent]
            Branding theme successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "branding/themes/default",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBrandingDefaultThemeResponseContent,
                    parse_obj_as(
                        type_=GetBrandingDefaultThemeResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, theme_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetBrandingThemeResponseContent]:
        """
        Retrieve branding theme.

        Parameters
        ----------
        theme_id : str
            The ID of the theme

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetBrandingThemeResponseContent]
            Branding theme successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"branding/themes/{encode_path_param(theme_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetBrandingThemeResponseContent,
                    parse_obj_as(
                        type_=GetBrandingThemeResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, theme_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Delete branding theme.

        Parameters
        ----------
        theme_id : str
            The ID of the theme

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"branding/themes/{encode_path_param(theme_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        theme_id: str,
        *,
        borders: BrandingThemeBorders,
        colors: BrandingThemeColors,
        fonts: BrandingThemeFonts,
        page_background: BrandingThemePageBackground,
        widget: BrandingThemeWidget,
        display_name: typing.Optional[str] = OMIT,
        identifiers: typing.Optional[BrandingThemeIdentifiers] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateBrandingThemeResponseContent]:
        """
        Update branding theme.

        Parameters
        ----------
        theme_id : str
            The ID of the theme

        borders : BrandingThemeBorders

        colors : BrandingThemeColors

        fonts : BrandingThemeFonts

        page_background : BrandingThemePageBackground

        widget : BrandingThemeWidget

        display_name : typing.Optional[str]
            Display Name

        identifiers : typing.Optional[BrandingThemeIdentifiers]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateBrandingThemeResponseContent]
            Branding settings successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"branding/themes/{encode_path_param(theme_id)}",
            method="PATCH",
            json={
                "borders": convert_and_respect_annotation_metadata(
                    object_=borders, annotation=BrandingThemeBorders, direction="write"
                ),
                "colors": convert_and_respect_annotation_metadata(
                    object_=colors, annotation=BrandingThemeColors, direction="write"
                ),
                "displayName": display_name,
                "fonts": convert_and_respect_annotation_metadata(
                    object_=fonts, annotation=BrandingThemeFonts, direction="write"
                ),
                "identifiers": convert_and_respect_annotation_metadata(
                    object_=identifiers, annotation=BrandingThemeIdentifiers, direction="write"
                ),
                "page_background": convert_and_respect_annotation_metadata(
                    object_=page_background, annotation=BrandingThemePageBackground, direction="write"
                ),
                "widget": convert_and_respect_annotation_metadata(
                    object_=widget, annotation=BrandingThemeWidget, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateBrandingThemeResponseContent,
                    parse_obj_as(
                        type_=UpdateBrandingThemeResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



from .conftest import get_client, verify_request_count


def test_guardian_factors_phone_get_message_types() -> None:
    """Test getMessageTypes endpoint with WireMock"""
    test_id = "guardian.factors.phone.get_message_types.0"
    client = get_client(test_id)
    client.guardian.factors.phone.get_message_types()
    verify_request_count(test_id, "GET", "/guardian/factors/phone/message-types", None, 1)


def test_guardian_factors_phone_set_message_types() -> None:
    """Test setMessageTypes endpoint with WireMock"""
    test_id = "guardian.factors.phone.set_message_types.0"
    client = get_client(test_id)
    client.guardian.factors.phone.set_message_types(
        message_types=["sms"],
    )
    verify_request_count(test_id, "PUT", "/guardian/factors/phone/message-types", None, 1)


def test_guardian_factors_phone_get_twilio_provider() -> None:
    """Test getTwilioProvider endpoint with WireMock"""
    test_id = "guardian.factors.phone.get_twilio_provider.0"
    client = get_client(test_id)
    client.guardian.factors.phone.get_twilio_provider()
    verify_request_count(test_id, "GET", "/guardian/factors/phone/providers/twilio", None, 1)


def test_guardian_factors_phone_set_twilio_provider() -> None:
    """Test setTwilioProvider endpoint with WireMock"""
    test_id = "guardian.factors.phone.set_twilio_provider.0"
    client = get_client(test_id)
    client.guardian.factors.phone.set_twilio_provider()
    verify_request_count(test_id, "PUT", "/guardian/factors/phone/providers/twilio", None, 1)


def test_guardian_factors_phone_get_selected_provider() -> None:
    """Test getSelectedProvider endpoint with WireMock"""
    test_id = "guardian.factors.phone.get_selected_provider.0"
    client = get_client(test_id)
    client.guardian.factors.phone.get_selected_provider()
    verify_request_count(test_id, "GET", "/guardian/factors/phone/selected-provider", None, 1)


def test_guardian_factors_phone_set_provider() -> None:
    """Test setProvider endpoint with WireMock"""
    test_id = "guardian.factors.phone.set_provider.0"
    client = get_client(test_id)
    client.guardian.factors.phone.set_provider(
        provider="auth0",
    )
    verify_request_count(test_id, "PUT", "/guardian/factors/phone/selected-provider", None, 1)


def test_guardian_factors_phone_get_templates() -> None:
    """Test getTemplates endpoint with WireMock"""
    test_id = "guardian.factors.phone.get_templates.0"
    client = get_client(test_id)
    client.guardian.factors.phone.get_templates()
    verify_request_count(test_id, "GET", "/guardian/factors/phone/templates", None, 1)


def test_guardian_factors_phone_set_templates() -> None:
    """Test setTemplates endpoint with WireMock"""
    test_id = "guardian.factors.phone.set_templates.0"
    client = get_client(test_id)
    client.guardian.factors.phone.set_templates(
        enrollment_message="enrollment_message",
        verification_message="verification_message",
    )
    verify_request_count(test_id, "PUT", "/guardian/factors/phone/templates", None, 1)



# This file was auto-generated by Fern from our API Definition.

from .connection_ext_groups import ConnectionExtGroups

ConnectionExtGroupsGoogleApps = ConnectionExtGroups
"""
Enables enrichment with Google group memberships (required for `ext_groups_extended`).
"""



# This file was auto-generated by Fern from our API Definition.

import typing

ResourceServerProofOfPossessionRequiredForEnum = typing.Union[
    typing.Literal["public_clients", "all_clients"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .event_stream_cloud_event_a_0_purpose_enum import EventStreamCloudEventA0PurposeEnum
from .event_stream_cloud_event_spec_version_enum import EventStreamCloudEventSpecVersionEnum
from .event_stream_cloud_event_user_created_cloud_event_type_enum import (
    EventStreamCloudEventUserCreatedCloudEventTypeEnum,
)
from .event_stream_cloud_event_user_created_data import EventStreamCloudEventUserCreatedData


class EventStreamCloudEventUserCreatedCloudEvent(UniversalBaseModel):
    """
    Represents an event that occurs when a user is created.
    """

    specversion: EventStreamCloudEventSpecVersionEnum
    type: EventStreamCloudEventUserCreatedCloudEventTypeEnum
    source: str = pydantic.Field()
    """
    The source of the event. This will take the form 'urn:auth0:<tenant>.<domain>'.
    """

    id: str = pydantic.Field()
    """
    A unique identifier for the event.
    """

    time: dt.datetime = pydantic.Field()
    """
    An ISO-8601 timestamp indicating when the event physically occurred.
    """

    data: EventStreamCloudEventUserCreatedData
    a_0_tenant: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="a0tenant"),
        pydantic.Field(alias="a0tenant", description="The auth0 tenant ID to which the event is associated."),
    ]
    """
    The auth0 tenant ID to which the event is associated.
    """

    a_0_stream: typing_extensions.Annotated[
        str,
        FieldMetadata(alias="a0stream"),
        pydantic.Field(
            alias="a0stream", description="The auth0 event stream ID of the stream the event was delivered on."
        ),
    ]
    """
    The auth0 event stream ID of the stream the event was delivered on.
    """

    a_0_purpose: typing_extensions.Annotated[
        typing.Optional[EventStreamCloudEventA0PurposeEnum],
        FieldMetadata(alias="a0purpose"),
        pydantic.Field(alias="a0purpose"),
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import encode_path_param
from ..core.pagination import AsyncPager, SyncPager
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.not_found_error import NotFoundError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.create_form_response_content import CreateFormResponseContent
from ..types.form_ending_node import FormEndingNode
from ..types.form_ending_node_nullable import FormEndingNodeNullable
from ..types.form_languages import FormLanguages
from ..types.form_languages_nullable import FormLanguagesNullable
from ..types.form_messages import FormMessages
from ..types.form_messages_nullable import FormMessagesNullable
from ..types.form_node_list import FormNodeList
from ..types.form_node_list_nullable import FormNodeListNullable
from ..types.form_start_node import FormStartNode
from ..types.form_start_node_nullable import FormStartNodeNullable
from ..types.form_style import FormStyle
from ..types.form_style_nullable import FormStyleNullable
from ..types.form_summary import FormSummary
from ..types.form_translations import FormTranslations
from ..types.form_translations_nullable import FormTranslationsNullable
from ..types.forms_request_parameters_hydrate_enum import FormsRequestParametersHydrateEnum
from ..types.get_form_response_content import GetFormResponseContent
from ..types.list_forms_offset_paginated_response_content import ListFormsOffsetPaginatedResponseContent
from ..types.update_form_response_content import UpdateFormResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawFormsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        hydrate: typing.Optional[
            typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[FormSummary, ListFormsOffsetPaginatedResponseContent]:
        """
        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        hydrate : typing.Optional[typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]]
            Query parameter to hydrate the response with additional data

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[FormSummary, ListFormsOffsetPaginatedResponseContent]
            Forms successfully retrieved.
        """
        page = page if page is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            "forms",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
                "include_totals": include_totals,
                "hydrate": hydrate,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListFormsOffsetPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListFormsOffsetPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.forms
                _has_next = len(_items or []) > 0
                _get_next = lambda: self.list(
                    page=page + 1,
                    per_page=per_page,
                    include_totals=include_totals,
                    hydrate=hydrate,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        name: str,
        messages: typing.Optional[FormMessages] = OMIT,
        languages: typing.Optional[FormLanguages] = OMIT,
        translations: typing.Optional[FormTranslations] = OMIT,
        nodes: typing.Optional[FormNodeList] = OMIT,
        start: typing.Optional[FormStartNode] = OMIT,
        ending: typing.Optional[FormEndingNode] = OMIT,
        style: typing.Optional[FormStyle] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateFormResponseContent]:
        """
        Parameters
        ----------
        name : str

        messages : typing.Optional[FormMessages]

        languages : typing.Optional[FormLanguages]

        translations : typing.Optional[FormTranslations]

        nodes : typing.Optional[FormNodeList]

        start : typing.Optional[FormStartNode]

        ending : typing.Optional[FormEndingNode]

        style : typing.Optional[FormStyle]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateFormResponseContent]
            Form successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "forms",
            method="POST",
            json={
                "name": name,
                "messages": convert_and_respect_annotation_metadata(
                    object_=messages, annotation=FormMessages, direction="write"
                ),
                "languages": convert_and_respect_annotation_metadata(
                    object_=languages, annotation=FormLanguages, direction="write"
                ),
                "translations": translations,
                "nodes": convert_and_respect_annotation_metadata(
                    object_=nodes, annotation=FormNodeList, direction="write"
                ),
                "start": convert_and_respect_annotation_metadata(
                    object_=start, annotation=FormStartNode, direction="write"
                ),
                "ending": convert_and_respect_annotation_metadata(
                    object_=ending, annotation=FormEndingNode, direction="write"
                ),
                "style": convert_and_respect_annotation_metadata(
                    object_=style, annotation=FormStyle, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateFormResponseContent,
                    parse_obj_as(
                        type_=CreateFormResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self,
        id: str,
        *,
        hydrate: typing.Optional[
            typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[GetFormResponseContent]:
        """
        Parameters
        ----------
        id : str
            The ID of the form to retrieve.

        hydrate : typing.Optional[typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]]
            Query parameter to hydrate the response with additional data

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetFormResponseContent]
            Form successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"forms/{encode_path_param(id)}",
            method="GET",
            params={
                "hydrate": hydrate,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetFormResponseContent,
                    parse_obj_as(
                        type_=GetFormResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        Parameters
        ----------
        id : str
            The ID of the form to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"forms/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        messages: typing.Optional[FormMessagesNullable] = OMIT,
        languages: typing.Optional[FormLanguagesNullable] = OMIT,
        translations: typing.Optional[FormTranslationsNullable] = OMIT,
        nodes: typing.Optional[FormNodeListNullable] = OMIT,
        start: typing.Optional[FormStartNodeNullable] = OMIT,
        ending: typing.Optional[FormEndingNodeNullable] = OMIT,
        style: typing.Optional[FormStyleNullable] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateFormResponseContent]:
        """
        Parameters
        ----------
        id : str
            The ID of the form to update.

        name : typing.Optional[str]

        messages : typing.Optional[FormMessagesNullable]

        languages : typing.Optional[FormLanguagesNullable]

        translations : typing.Optional[FormTranslationsNullable]

        nodes : typing.Optional[FormNodeListNullable]

        start : typing.Optional[FormStartNodeNullable]

        ending : typing.Optional[FormEndingNodeNullable]

        style : typing.Optional[FormStyleNullable]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateFormResponseContent]
            Form successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"forms/{encode_path_param(id)}",
            method="PATCH",
            json={
                "name": name,
                "messages": convert_and_respect_annotation_metadata(
                    object_=messages, annotation=FormMessagesNullable, direction="write"
                ),
                "languages": convert_and_respect_annotation_metadata(
                    object_=languages, annotation=FormLanguagesNullable, direction="write"
                ),
                "translations": translations,
                "nodes": convert_and_respect_annotation_metadata(
                    object_=nodes, annotation=FormNodeListNullable, direction="write"
                ),
                "start": convert_and_respect_annotation_metadata(
                    object_=start, annotation=FormStartNodeNullable, direction="write"
                ),
                "ending": convert_and_respect_annotation_metadata(
                    object_=ending, annotation=FormEndingNodeNullable, direction="write"
                ),
                "style": convert_and_respect_annotation_metadata(
                    object_=style, annotation=FormStyleNullable, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateFormResponseContent,
                    parse_obj_as(
                        type_=UpdateFormResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawFormsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        hydrate: typing.Optional[
            typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[FormSummary, ListFormsOffsetPaginatedResponseContent]:
        """
        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Defaults to 50.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        hydrate : typing.Optional[typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]]
            Query parameter to hydrate the response with additional data

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[FormSummary, ListFormsOffsetPaginatedResponseContent]
            Forms successfully retrieved.
        """
        page = page if page is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            "forms",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
                "include_totals": include_totals,
                "hydrate": hydrate,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListFormsOffsetPaginatedResponseContent,
                    parse_obj_as(
                        type_=ListFormsOffsetPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.forms
                _has_next = len(_items or []) > 0

                async def _get_next():
                    return await self.list(
                        page=page + 1,
                        per_page=per_page,
                        include_totals=include_totals,
                        hydrate=hydrate,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        name: str,
        messages: typing.Optional[FormMessages] = OMIT,
        languages: typing.Optional[FormLanguages] = OMIT,
        translations: typing.Optional[FormTranslations] = OMIT,
        nodes: typing.Optional[FormNodeList] = OMIT,
        start: typing.Optional[FormStartNode] = OMIT,
        ending: typing.Optional[FormEndingNode] = OMIT,
        style: typing.Optional[FormStyle] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateFormResponseContent]:
        """
        Parameters
        ----------
        name : str

        messages : typing.Optional[FormMessages]

        languages : typing.Optional[FormLanguages]

        translations : typing.Optional[FormTranslations]

        nodes : typing.Optional[FormNodeList]

        start : typing.Optional[FormStartNode]

        ending : typing.Optional[FormEndingNode]

        style : typing.Optional[FormStyle]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateFormResponseContent]
            Form successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "forms",
            method="POST",
            json={
                "name": name,
                "messages": convert_and_respect_annotation_metadata(
                    object_=messages, annotation=FormMessages, direction="write"
                ),
                "languages": convert_and_respect_annotation_metadata(
                    object_=languages, annotation=FormLanguages, direction="write"
                ),
                "translations": translations,
                "nodes": convert_and_respect_annotation_metadata(
                    object_=nodes, annotation=FormNodeList, direction="write"
                ),
                "start": convert_and_respect_annotation_metadata(
                    object_=start, annotation=FormStartNode, direction="write"
                ),
                "ending": convert_and_respect_annotation_metadata(
                    object_=ending, annotation=FormEndingNode, direction="write"
                ),
                "style": convert_and_respect_annotation_metadata(
                    object_=style, annotation=FormStyle, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateFormResponseContent,
                    parse_obj_as(
                        type_=CreateFormResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self,
        id: str,
        *,
        hydrate: typing.Optional[
            typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[GetFormResponseContent]:
        """
        Parameters
        ----------
        id : str
            The ID of the form to retrieve.

        hydrate : typing.Optional[typing.Union[FormsRequestParametersHydrateEnum, typing.Sequence[FormsRequestParametersHydrateEnum]]]
            Query parameter to hydrate the response with additional data

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetFormResponseContent]
            Form successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"forms/{encode_path_param(id)}",
            method="GET",
            params={
                "hydrate": hydrate,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetFormResponseContent,
                    parse_obj_as(
                        type_=GetFormResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Parameters
        ----------
        id : str
            The ID of the form to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"forms/{encode_path_param(id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        messages: typing.Optional[FormMessagesNullable] = OMIT,
        languages: typing.Optional[FormLanguagesNullable] = OMIT,
        translations: typing.Optional[FormTranslationsNullable] = OMIT,
        nodes: typing.Optional[FormNodeListNullable] = OMIT,
        start: typing.Optional[FormStartNodeNullable] = OMIT,
        ending: typing.Optional[FormEndingNodeNullable] = OMIT,
        style: typing.Optional[FormStyleNullable] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateFormResponseContent]:
        """
        Parameters
        ----------
        id : str
            The ID of the form to update.

        name : typing.Optional[str]

        messages : typing.Optional[FormMessagesNullable]

        languages : typing.Optional[FormLanguagesNullable]

        translations : typing.Optional[FormTranslationsNullable]

        nodes : typing.Optional[FormNodeListNullable]

        start : typing.Optional[FormStartNodeNullable]

        ending : typing.Optional[FormEndingNodeNullable]

        style : typing.Optional[FormStyleNullable]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateFormResponseContent]
            Form successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"forms/{encode_path_param(id)}",
            method="PATCH",
            json={
                "name": name,
                "messages": convert_and_respect_annotation_metadata(
                    object_=messages, annotation=FormMessagesNullable, direction="write"
                ),
                "languages": convert_and_respect_annotation_metadata(
                    object_=languages, annotation=FormLanguagesNullable, direction="write"
                ),
                "translations": translations,
                "nodes": convert_and_respect_annotation_metadata(
                    object_=nodes, annotation=FormNodeListNullable, direction="write"
                ),
                "start": convert_and_respect_annotation_metadata(
                    object_=start, annotation=FormStartNodeNullable, direction="write"
                ),
                "ending": convert_and_respect_annotation_metadata(
                    object_=ending, annotation=FormEndingNodeNullable, direction="write"
                ),
                "style": convert_and_respect_annotation_metadata(
                    object_=style, annotation=FormStyleNullable, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateFormResponseContent,
                    parse_obj_as(
                        type_=UpdateFormResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from . import (
        authentication_methods,
        authenticators,
        connected_accounts,
        effective_permissions,
        effective_roles,
        enrollments,
        groups,
        identities,
        logs,
        multifactor,
        organizations,
        permissions,
        refresh_token,
        risk_assessments,
        roles,
        sessions,
    )
_dynamic_imports: typing.Dict[str, str] = {
    "authentication_methods": ".authentication_methods",
    "authenticators": ".authenticators",
    "connected_accounts": ".connected_accounts",
    "effective_permissions": ".effective_permissions",
    "effective_roles": ".effective_roles",
    "enrollments": ".enrollments",
    "groups": ".groups",
    "identities": ".identities",
    "logs": ".logs",
    "multifactor": ".multifactor",
    "organizations": ".organizations",
    "permissions": ".permissions",
    "refresh_token": ".refresh_token",
    "risk_assessments": ".risk_assessments",
    "roles": ".roles",
    "sessions": ".sessions",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "authentication_methods",
    "authenticators",
    "connected_accounts",
    "effective_permissions",
    "effective_roles",
    "enrollments",
    "groups",
    "identities",
    "logs",
    "multifactor",
    "organizations",
    "permissions",
    "refresh_token",
    "risk_assessments",
    "roles",
    "sessions",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .group_member_type_enum import GroupMemberTypeEnum
from .group_type_enum import GroupTypeEnum


class GroupMember(UniversalBaseModel):
    """
    Represents the metadata of a group membership.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the member.
    """

    member_type: typing.Optional[GroupMemberTypeEnum] = None
    type: typing.Optional[GroupTypeEnum] = None
    connection_id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Identifier for the connection this group belongs to (if a connection group).
    """

    created_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    Timestamp of when the membership was created.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .create_flows_vault_connection_http_api_key import CreateFlowsVaultConnectionHttpApiKey
from .create_flows_vault_connection_http_basic_auth import CreateFlowsVaultConnectionHttpBasicAuth
from .create_flows_vault_connection_http_bearer import CreateFlowsVaultConnectionHttpBearer
from .create_flows_vault_connection_http_oauth_client_credentials import (
    CreateFlowsVaultConnectionHttpOauthClientCredentials,
)
from .create_flows_vault_connection_http_uninitialized import CreateFlowsVaultConnectionHttpUninitialized

CreateFlowsVaultConnectionHttp = typing.Union[
    CreateFlowsVaultConnectionHttpBearer,
    CreateFlowsVaultConnectionHttpBasicAuth,
    CreateFlowsVaultConnectionHttpApiKey,
    CreateFlowsVaultConnectionHttpOauthClientCredentials,
    CreateFlowsVaultConnectionHttpUninitialized,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .domain_verification_method import DomainVerificationMethod
from .domain_verification_status_enum import DomainVerificationStatusEnum


class DomainVerification(UniversalBaseModel):
    """
    Domain verification settings.
    """

    methods: typing.Optional[typing.List[DomainVerificationMethod]] = pydantic.Field(default=None)
    """
    Domain verification methods.
    """

    status: typing.Optional[DomainVerificationStatusEnum] = None
    error_msg: typing.Optional[str] = pydantic.Field(default=None)
    """
    The user0-friendly error message in case of failed verification. This field is relevant only for Custom Domains with Auth0-Managed Certificates.
    """

    last_verified_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The date and time when the custom domain was last verified. This field is relevant only for Custom Domains with Auth0-Managed Certificates.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.parse_error import ParsingError
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.too_many_requests_error import TooManyRequestsError
from ..errors.unauthorized_error import UnauthorizedError
from ..types.get_settings_response_content import GetSettingsResponseContent
from ..types.universal_login_experience_enum import UniversalLoginExperienceEnum
from ..types.update_settings_response_content import UpdateSettingsResponseContent
from pydantic import ValidationError

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawPromptsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_settings(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[GetSettingsResponseContent]:
        """
        Retrieve details of the Universal Login configuration of your tenant. This includes the <a href="https://auth0.com/docs/authenticate/login/auth0-universal-login/identifier-first">Identifier First Authentication</a> and <a href="https://auth0.com/docs/secure/multi-factor-authentication/fido-authentication-with-webauthn/configure-webauthn-device-biometrics-for-mfa">WebAuthn with Device Biometrics for MFA</a> features.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[GetSettingsResponseContent]
            Prompt settings successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            "prompts",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSettingsResponseContent,
                    parse_obj_as(
                        type_=GetSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update_settings(
        self,
        *,
        universal_login_experience: typing.Optional[UniversalLoginExperienceEnum] = OMIT,
        identifier_first: typing.Optional[bool] = OMIT,
        webauthn_platform_first_factor: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[UpdateSettingsResponseContent]:
        """
        Update the Universal Login configuration of your tenant. This includes the <a href="https://auth0.com/docs/authenticate/login/auth0-universal-login/identifier-first">Identifier First Authentication</a> and <a href="https://auth0.com/docs/secure/multi-factor-authentication/fido-authentication-with-webauthn/configure-webauthn-device-biometrics-for-mfa">WebAuthn with Device Biometrics for MFA</a> features.

        Parameters
        ----------
        universal_login_experience : typing.Optional[UniversalLoginExperienceEnum]

        identifier_first : typing.Optional[bool]
            Whether identifier first is enabled or not

        webauthn_platform_first_factor : typing.Optional[bool]
            Use WebAuthn with Device Biometrics as the first authentication factor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[UpdateSettingsResponseContent]
            Prompts settings successfully updated.
        """
        _response = self._client_wrapper.httpx_client.request(
            "prompts",
            method="PATCH",
            json={
                "universal_login_experience": universal_login_experience,
                "identifier_first": identifier_first,
                "webauthn_platform_first_factor": webauthn_platform_first_factor,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateSettingsResponseContent,
                    parse_obj_as(
                        type_=UpdateSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawPromptsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_settings(
        self, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[GetSettingsResponseContent]:
        """
        Retrieve details of the Universal Login configuration of your tenant. This includes the <a href="https://auth0.com/docs/authenticate/login/auth0-universal-login/identifier-first">Identifier First Authentication</a> and <a href="https://auth0.com/docs/secure/multi-factor-authentication/fido-authentication-with-webauthn/configure-webauthn-device-biometrics-for-mfa">WebAuthn with Device Biometrics for MFA</a> features.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[GetSettingsResponseContent]
            Prompt settings successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "prompts",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    GetSettingsResponseContent,
                    parse_obj_as(
                        type_=GetSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update_settings(
        self,
        *,
        universal_login_experience: typing.Optional[UniversalLoginExperienceEnum] = OMIT,
        identifier_first: typing.Optional[bool] = OMIT,
        webauthn_platform_first_factor: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[UpdateSettingsResponseContent]:
        """
        Update the Universal Login configuration of your tenant. This includes the <a href="https://auth0.com/docs/authenticate/login/auth0-universal-login/identifier-first">Identifier First Authentication</a> and <a href="https://auth0.com/docs/secure/multi-factor-authentication/fido-authentication-with-webauthn/configure-webauthn-device-biometrics-for-mfa">WebAuthn with Device Biometrics for MFA</a> features.

        Parameters
        ----------
        universal_login_experience : typing.Optional[UniversalLoginExperienceEnum]

        identifier_first : typing.Optional[bool]
            Whether identifier first is enabled or not

        webauthn_platform_first_factor : typing.Optional[bool]
            Use WebAuthn with Device Biometrics as the first authentication factor

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[UpdateSettingsResponseContent]
            Prompts settings successfully updated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "prompts",
            method="PATCH",
            json={
                "universal_login_experience": universal_login_experience,
                "identifier_first": identifier_first,
                "webauthn_platform_first_factor": webauthn_platform_first_factor,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    UpdateSettingsResponseContent,
                    parse_obj_as(
                        type_=UpdateSettingsResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.get_partials_response_content import GetPartialsResponseContent
from ...types.partial_groups_enum import PartialGroupsEnum
from ...types.set_partials_request_content import SetPartialsRequestContent
from .raw_client import AsyncRawPartialsClient, RawPartialsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class PartialsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawPartialsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawPartialsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawPartialsClient
        """
        return self._raw_client

    def get(
        self, prompt: PartialGroupsEnum, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetPartialsResponseContent:
        """
        Get template partials for a prompt

        Parameters
        ----------
        prompt : PartialGroupsEnum
            Name of the prompt.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPartialsResponseContent
            Prompt partials successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.prompts.partials.get(
            prompt="login",
        )
        """
        _response = self._raw_client.get(prompt, request_options=request_options)
        return _response.data

    def set(
        self,
        prompt: PartialGroupsEnum,
        *,
        request: SetPartialsRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Set template partials for a prompt

        Parameters
        ----------
        prompt : PartialGroupsEnum
            Name of the prompt.

        request : SetPartialsRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.prompts.partials.set(
            prompt="login",
            request={"key": "value"},
        )
        """
        _response = self._raw_client.set(prompt, request=request, request_options=request_options)
        return _response.data


class AsyncPartialsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawPartialsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawPartialsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawPartialsClient
        """
        return self._raw_client

    async def get(
        self, prompt: PartialGroupsEnum, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetPartialsResponseContent:
        """
        Get template partials for a prompt

        Parameters
        ----------
        prompt : PartialGroupsEnum
            Name of the prompt.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetPartialsResponseContent
            Prompt partials successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.prompts.partials.get(
                prompt="login",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(prompt, request_options=request_options)
        return _response.data

    async def set(
        self,
        prompt: PartialGroupsEnum,
        *,
        request: SetPartialsRequestContent,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Set template partials for a prompt

        Parameters
        ----------
        prompt : PartialGroupsEnum
            Name of the prompt.

        request : SetPartialsRequestContent

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.prompts.partials.set(
                prompt="login",
                request={"key": "value"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.set(prompt, request=request, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .csp_policy import CspPolicy

CspPolicies = typing.List[CspPolicy]
"""
Array of CSP policies (enforcing and/or reporting).
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.get_network_acls_response_content import GetNetworkAclsResponseContent
from ..types.list_network_acls_offset_paginated_response_content import ListNetworkAclsOffsetPaginatedResponseContent
from ..types.network_acl_rule import NetworkAclRule
from ..types.network_acls_response_content import NetworkAclsResponseContent
from ..types.set_network_acls_response_content import SetNetworkAclsResponseContent
from ..types.update_network_acl_response_content import UpdateNetworkAclResponseContent
from .raw_client import AsyncRawNetworkAclsClient, RawNetworkAclsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class NetworkAclsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawNetworkAclsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawNetworkAclsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawNetworkAclsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[NetworkAclsResponseContent, ListNetworkAclsOffsetPaginatedResponseContent]:
        """
        Get all access control list entries for your client.

        Parameters
        ----------
        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            The amount of results per page.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[NetworkAclsResponseContent, ListNetworkAclsOffsetPaginatedResponseContent]
            Network access control list successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.network_acls.list(
            page=1,
            per_page=1,
            include_totals=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            page=page, per_page=per_page, include_totals=include_totals, request_options=request_options
        )

    def create(
        self,
        *,
        description: str,
        active: bool,
        rule: NetworkAclRule,
        priority: typing.Optional[float] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Create a new access control list for your client.

        Parameters
        ----------
        description : str

        active : bool
            Indicates whether or not this access control list is actively being used

        rule : NetworkAclRule

        priority : typing.Optional[float]
            Indicates the order in which the ACL will be evaluated relative to other ACL rules.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0, NetworkAclAction, NetworkAclRule

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.network_acls.create(
            description="description",
            active=True,
            rule=NetworkAclRule(
                action=NetworkAclAction(),
                scope="management",
            ),
        )
        """
        _response = self._raw_client.create(
            description=description, active=active, rule=rule, priority=priority, request_options=request_options
        )
        return _response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> GetNetworkAclsResponseContent:
        """
        Get a specific access control list entry for your client.

        Parameters
        ----------
        id : str
            The id of the access control list to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetNetworkAclsResponseContent
            Network access control list successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.network_acls.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def set(
        self,
        id: str,
        *,
        description: str,
        active: bool,
        rule: NetworkAclRule,
        priority: typing.Optional[float] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SetNetworkAclsResponseContent:
        """
        Update existing access control list for your client.

        Parameters
        ----------
        id : str
            The id of the ACL to update.

        description : str

        active : bool
            Indicates whether or not this access control list is actively being used

        rule : NetworkAclRule

        priority : typing.Optional[float]
            Indicates the order in which the ACL will be evaluated relative to other ACL rules.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetNetworkAclsResponseContent
            Network ACL properties successfully updated

        Examples
        --------
        from auth0 import Auth0, NetworkAclAction, NetworkAclRule

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.network_acls.set(
            id="id",
            description="description",
            active=True,
            rule=NetworkAclRule(
                action=NetworkAclAction(),
                scope="management",
            ),
        )
        """
        _response = self._raw_client.set(
            id, description=description, active=active, rule=rule, priority=priority, request_options=request_options
        )
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete existing access control list for your client.

        Parameters
        ----------
        id : str
            The id of the ACL to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.network_acls.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        description: typing.Optional[str] = OMIT,
        active: typing.Optional[bool] = OMIT,
        priority: typing.Optional[float] = OMIT,
        rule: typing.Optional[NetworkAclRule] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateNetworkAclResponseContent:
        """
        Update existing access control list for your client.

        Parameters
        ----------
        id : str
            The id of the ACL to update.

        description : typing.Optional[str]

        active : typing.Optional[bool]
            Indicates whether or not this access control list is actively being used

        priority : typing.Optional[float]
            Indicates the order in which the ACL will be evaluated relative to other ACL rules.

        rule : typing.Optional[NetworkAclRule]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateNetworkAclResponseContent
            Network ACL properties successfully updated

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.network_acls.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id, description=description, active=active, priority=priority, rule=rule, request_options=request_options
        )
        return _response.data


class AsyncNetworkAclsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawNetworkAclsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawNetworkAclsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawNetworkAclsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[NetworkAclsResponseContent, ListNetworkAclsOffsetPaginatedResponseContent]:
        """
        Get all access control list entries for your client.

        Parameters
        ----------
        page : typing.Optional[int]
            Use this field to request a specific page of the list results.

        per_page : typing.Optional[int]
            The amount of results per page.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[NetworkAclsResponseContent, ListNetworkAclsOffsetPaginatedResponseContent]
            Network access control list successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.network_acls.list(
                page=1,
                per_page=1,
                include_totals=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            page=page, per_page=per_page, include_totals=include_totals, request_options=request_options
        )

    async def create(
        self,
        *,
        description: str,
        active: bool,
        rule: NetworkAclRule,
        priority: typing.Optional[float] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Create a new access control list for your client.

        Parameters
        ----------
        description : str

        active : bool
            Indicates whether or not this access control list is actively being used

        rule : NetworkAclRule

        priority : typing.Optional[float]
            Indicates the order in which the ACL will be evaluated relative to other ACL rules.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, NetworkAclAction, NetworkAclRule

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.network_acls.create(
                description="description",
                active=True,
                rule=NetworkAclRule(
                    action=NetworkAclAction(),
                    scope="management",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            description=description, active=active, rule=rule, priority=priority, request_options=request_options
        )
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetNetworkAclsResponseContent:
        """
        Get a specific access control list entry for your client.

        Parameters
        ----------
        id : str
            The id of the access control list to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetNetworkAclsResponseContent
            Network access control list successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.network_acls.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def set(
        self,
        id: str,
        *,
        description: str,
        active: bool,
        rule: NetworkAclRule,
        priority: typing.Optional[float] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SetNetworkAclsResponseContent:
        """
        Update existing access control list for your client.

        Parameters
        ----------
        id : str
            The id of the ACL to update.

        description : str

        active : bool
            Indicates whether or not this access control list is actively being used

        rule : NetworkAclRule

        priority : typing.Optional[float]
            Indicates the order in which the ACL will be evaluated relative to other ACL rules.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SetNetworkAclsResponseContent
            Network ACL properties successfully updated

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, NetworkAclAction, NetworkAclRule

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.network_acls.set(
                id="id",
                description="description",
                active=True,
                rule=NetworkAclRule(
                    action=NetworkAclAction(),
                    scope="management",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.set(
            id, description=description, active=active, rule=rule, priority=priority, request_options=request_options
        )
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete existing access control list for your client.

        Parameters
        ----------
        id : str
            The id of the ACL to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.network_acls.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        description: typing.Optional[str] = OMIT,
        active: typing.Optional[bool] = OMIT,
        priority: typing.Optional[float] = OMIT,
        rule: typing.Optional[NetworkAclRule] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateNetworkAclResponseContent:
        """
        Update existing access control list for your client.

        Parameters
        ----------
        id : str
            The id of the ACL to update.

        description : typing.Optional[str]

        active : typing.Optional[bool]
            Indicates whether or not this access control list is actively being used

        priority : typing.Optional[float]
            Indicates the order in which the ACL will be evaluated relative to other ACL rules.

        rule : typing.Optional[NetworkAclRule]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateNetworkAclResponseContent
            Network ACL properties successfully updated

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.network_acls.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id, description=description, active=active, priority=priority, rule=rule, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

from . import management
from . import authentication
from auth0.management.version import __version__

__all__ = ["management", "authentication", "__version__"]



# This file was auto-generated by Fern from our API Definition.

import typing

from .create_flows_vault_connection_stripe_key_pair import CreateFlowsVaultConnectionStripeKeyPair
from .create_flows_vault_connection_stripe_oauth_code import CreateFlowsVaultConnectionStripeOauthCode
from .create_flows_vault_connection_stripe_uninitialized import CreateFlowsVaultConnectionStripeUninitialized

CreateFlowsVaultConnectionStripe = typing.Union[
    CreateFlowsVaultConnectionStripeKeyPair,
    CreateFlowsVaultConnectionStripeOauthCode,
    CreateFlowsVaultConnectionStripeUninitialized,
]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.create_token_exchange_profile_response_content import CreateTokenExchangeProfileResponseContent
from ..types.get_token_exchange_profile_response_content import GetTokenExchangeProfileResponseContent
from ..types.list_token_exchange_profile_response_content import ListTokenExchangeProfileResponseContent
from ..types.token_exchange_profile_response_content import TokenExchangeProfileResponseContent
from ..types.token_exchange_profile_type_enum import TokenExchangeProfileTypeEnum
from .raw_client import AsyncRawTokenExchangeProfilesClient, RawTokenExchangeProfilesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TokenExchangeProfilesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTokenExchangeProfilesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTokenExchangeProfilesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTokenExchangeProfilesClient
        """
        return self._raw_client

    def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[TokenExchangeProfileResponseContent, ListTokenExchangeProfileResponseContent]:
        """
        Retrieve a list of all Token Exchange Profiles available in your tenant.

        By using this feature, you agree to the applicable Free Trial terms in [Okta’s Master Subscription Agreement](https://www.okta.com/legal/). It is your responsibility to securely validate the user’s subject_token. See [User Guide](https://auth0.com/docs/authenticate/custom-token-exchange) for more details.

        This endpoint supports Checkpoint pagination. To search by checkpoint, use the following parameters:

        - `from`: Optional id from which to start selection.
        - `take`: The total amount of entries to retrieve when using the from parameter. Defaults to 50.

        **Note**: The first time you call this endpoint using checkpoint pagination, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no pages are remaining.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[TokenExchangeProfileResponseContent, ListTokenExchangeProfileResponseContent]
            Token Exchange Profile successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.token_exchange_profiles.list(
            from_="from",
            take=1,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(from_=from_, take=take, request_options=request_options)

    def create(
        self,
        *,
        name: str,
        subject_token_type: str,
        action_id: str,
        type: TokenExchangeProfileTypeEnum,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateTokenExchangeProfileResponseContent:
        """
        Create a new Token Exchange Profile within your tenant.

        By using this feature, you agree to the applicable Free Trial terms in [Okta’s Master Subscription Agreement](https://www.okta.com/legal/). It is your responsibility to securely validate the user’s subject_token. See [User Guide](https://auth0.com/docs/authenticate/custom-token-exchange) for more details.

        Parameters
        ----------
        name : str
            Friendly name of this profile.

        subject_token_type : str
            Subject token type for this profile. When receiving a token exchange request on the Authentication API, the corresponding token exchange profile with a matching subject_token_type will be executed. This must be a URI.

        action_id : str
            The ID of the Custom Token Exchange action to execute for this profile, in order to validate the subject_token. The action must use the custom-token-exchange trigger.

        type : TokenExchangeProfileTypeEnum

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateTokenExchangeProfileResponseContent
            Token exchange profile successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.token_exchange_profiles.create(
            name="name",
            subject_token_type="subject_token_type",
            action_id="action_id",
            type="custom_authentication",
        )
        """
        _response = self._raw_client.create(
            name=name,
            subject_token_type=subject_token_type,
            action_id=action_id,
            type=type,
            request_options=request_options,
        )
        return _response.data

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTokenExchangeProfileResponseContent:
        """
        Retrieve details about a single Token Exchange Profile specified by ID.

        By using this feature, you agree to the applicable Free Trial terms in [Okta’s Master Subscription Agreement](https://www.okta.com/legal/). It is your responsibility to securely validate the user’s subject_token. See [User Guide](https://auth0.com/docs/authenticate/custom-token-exchange) for more details.

        Parameters
        ----------
        id : str
            ID of the Token Exchange Profile to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTokenExchangeProfileResponseContent
            Token Exchange Profile successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.token_exchange_profiles.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a Token Exchange Profile within your tenant.

        By using this feature, you agree to the applicable Free Trial terms in [Okta's Master Subscription Agreement](https://www.okta.com/legal/). It is your responsibility to securely validate the user's subject_token. See [User Guide](https://auth0.com/docs/authenticate/custom-token-exchange) for more details.

        Parameters
        ----------
        id : str
            ID of the Token Exchange Profile to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.token_exchange_profiles.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        subject_token_type: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Update a Token Exchange Profile within your tenant.

        By using this feature, you agree to the applicable Free Trial terms in [Okta's Master Subscription Agreement](https://www.okta.com/legal/). It is your responsibility to securely validate the user's subject_token. See [User Guide](https://auth0.com/docs/authenticate/custom-token-exchange) for more details.

        Parameters
        ----------
        id : str
            ID of the Token Exchange Profile to update.

        name : typing.Optional[str]
            Friendly name of this profile.

        subject_token_type : typing.Optional[str]
            Subject token type for this profile. When receiving a token exchange request on the Authentication API, the corresponding token exchange profile with a matching subject_token_type will be executed. This must be a URI.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.token_exchange_profiles.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id, name=name, subject_token_type=subject_token_type, request_options=request_options
        )
        return _response.data


class AsyncTokenExchangeProfilesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTokenExchangeProfilesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTokenExchangeProfilesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTokenExchangeProfilesClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        from_: typing.Optional[str] = None,
        take: typing.Optional[int] = 50,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[TokenExchangeProfileResponseContent, ListTokenExchangeProfileResponseContent]:
        """
        Retrieve a list of all Token Exchange Profiles available in your tenant.

        By using this feature, you agree to the applicable Free Trial terms in [Okta’s Master Subscription Agreement](https://www.okta.com/legal/). It is your responsibility to securely validate the user’s subject_token. See [User Guide](https://auth0.com/docs/authenticate/custom-token-exchange) for more details.

        This endpoint supports Checkpoint pagination. To search by checkpoint, use the following parameters:

        - `from`: Optional id from which to start selection.
        - `take`: The total amount of entries to retrieve when using the from parameter. Defaults to 50.

        **Note**: The first time you call this endpoint using checkpoint pagination, omit the `from` parameter. If there are more results, a `next` value is included in the response. You can use this for subsequent API calls. When `next` is no longer included in the response, no pages are remaining.

        Parameters
        ----------
        from_ : typing.Optional[str]
            Optional Id from which to start selection.

        take : typing.Optional[int]
            Number of results per page. Defaults to 50.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[TokenExchangeProfileResponseContent, ListTokenExchangeProfileResponseContent]
            Token Exchange Profile successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.token_exchange_profiles.list(
                from_="from",
                take=1,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(from_=from_, take=take, request_options=request_options)

    async def create(
        self,
        *,
        name: str,
        subject_token_type: str,
        action_id: str,
        type: TokenExchangeProfileTypeEnum,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateTokenExchangeProfileResponseContent:
        """
        Create a new Token Exchange Profile within your tenant.

        By using this feature, you agree to the applicable Free Trial terms in [Okta’s Master Subscription Agreement](https://www.okta.com/legal/). It is your responsibility to securely validate the user’s subject_token. See [User Guide](https://auth0.com/docs/authenticate/custom-token-exchange) for more details.

        Parameters
        ----------
        name : str
            Friendly name of this profile.

        subject_token_type : str
            Subject token type for this profile. When receiving a token exchange request on the Authentication API, the corresponding token exchange profile with a matching subject_token_type will be executed. This must be a URI.

        action_id : str
            The ID of the Custom Token Exchange action to execute for this profile, in order to validate the subject_token. The action must use the custom-token-exchange trigger.

        type : TokenExchangeProfileTypeEnum

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateTokenExchangeProfileResponseContent
            Token exchange profile successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.token_exchange_profiles.create(
                name="name",
                subject_token_type="subject_token_type",
                action_id="action_id",
                type="custom_authentication",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            subject_token_type=subject_token_type,
            action_id=action_id,
            type=type,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetTokenExchangeProfileResponseContent:
        """
        Retrieve details about a single Token Exchange Profile specified by ID.

        By using this feature, you agree to the applicable Free Trial terms in [Okta’s Master Subscription Agreement](https://www.okta.com/legal/). It is your responsibility to securely validate the user’s subject_token. See [User Guide](https://auth0.com/docs/authenticate/custom-token-exchange) for more details.

        Parameters
        ----------
        id : str
            ID of the Token Exchange Profile to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetTokenExchangeProfileResponseContent
            Token Exchange Profile successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.token_exchange_profiles.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a Token Exchange Profile within your tenant.

        By using this feature, you agree to the applicable Free Trial terms in [Okta's Master Subscription Agreement](https://www.okta.com/legal/). It is your responsibility to securely validate the user's subject_token. See [User Guide](https://auth0.com/docs/authenticate/custom-token-exchange) for more details.

        Parameters
        ----------
        id : str
            ID of the Token Exchange Profile to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.token_exchange_profiles.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        name: typing.Optional[str] = OMIT,
        subject_token_type: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Update a Token Exchange Profile within your tenant.

        By using this feature, you agree to the applicable Free Trial terms in [Okta's Master Subscription Agreement](https://www.okta.com/legal/). It is your responsibility to securely validate the user's subject_token. See [User Guide](https://auth0.com/docs/authenticate/custom-token-exchange) for more details.

        Parameters
        ----------
        id : str
            ID of the Token Exchange Profile to update.

        name : typing.Optional[str]
            Friendly name of this profile.

        subject_token_type : typing.Optional[str]
            Subject token type for this profile. When receiving a token exchange request on the Authentication API, the corresponding token exchange profile with a matching subject_token_type will be executed. This must be a URI.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.token_exchange_profiles.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id, name=name, subject_token_type=subject_token_type, request_options=request_options
        )
        return _response.data



from .conftest import get_client, verify_request_count


def test_hooks_secrets_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "hooks.secrets.get.0"
    client = get_client(test_id)
    client.hooks.secrets.get(
        id="id",
    )
    verify_request_count(test_id, "GET", "/hooks/id/secrets", None, 1)


def test_hooks_secrets_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "hooks.secrets.create.0"
    client = get_client(test_id)
    client.hooks.secrets.create(
        id="id",
        request={"key": "value"},
    )
    verify_request_count(test_id, "POST", "/hooks/id/secrets", None, 1)


def test_hooks_secrets_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "hooks.secrets.delete.0"
    client = get_client(test_id)
    client.hooks.secrets.delete(
        id="id",
        request=["string"],
    )
    verify_request_count(test_id, "DELETE", "/hooks/id/secrets", None, 1)


def test_hooks_secrets_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "hooks.secrets.update.0"
    client = get_client(test_id)
    client.hooks.secrets.update(
        id="id",
        request={"key": "value"},
    )
    verify_request_count(test_id, "PATCH", "/hooks/id/secrets", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_cloud_event_connection_deleted_object_0 import EventStreamCloudEventConnectionDeletedObject0
from .event_stream_cloud_event_connection_deleted_object_1 import EventStreamCloudEventConnectionDeletedObject1
from .event_stream_cloud_event_connection_deleted_object_2 import EventStreamCloudEventConnectionDeletedObject2
from .event_stream_cloud_event_connection_deleted_object_3 import EventStreamCloudEventConnectionDeletedObject3
from .event_stream_cloud_event_connection_deleted_object_4 import EventStreamCloudEventConnectionDeletedObject4
from .event_stream_cloud_event_connection_deleted_object_5 import EventStreamCloudEventConnectionDeletedObject5
from .event_stream_cloud_event_connection_deleted_object_6 import EventStreamCloudEventConnectionDeletedObject6
from .event_stream_cloud_event_connection_deleted_object_7 import EventStreamCloudEventConnectionDeletedObject7

EventStreamCloudEventConnectionDeletedObject = typing.Union[
    EventStreamCloudEventConnectionDeletedObject0,
    EventStreamCloudEventConnectionDeletedObject1,
    EventStreamCloudEventConnectionDeletedObject2,
    EventStreamCloudEventConnectionDeletedObject3,
    EventStreamCloudEventConnectionDeletedObject4,
    EventStreamCloudEventConnectionDeletedObject5,
    EventStreamCloudEventConnectionDeletedObject6,
    EventStreamCloudEventConnectionDeletedObject7,
]



# This file was auto-generated by Fern from our API Definition.

from .connection_https_url_with_http_fallback_255 import ConnectionHttpsUrlWithHttpFallback255

ConnectionGatewayUrlSms = ConnectionHttpsUrlWithHttpFallback255
"""
The URL of your SMS gateway. Auth0 must be able to reach this URL for it to use your gateway to send messages on your behalf.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from .bot_detection_ip_address_or_cidr_block import BotDetectionIpAddressOrCidrBlock

BotDetectionAllowlist = typing.List[BotDetectionIpAddressOrCidrBlock]
"""
List of IP addresses or CIDR blocks to allowlist
"""



# This file was auto-generated by Fern from our API Definition.

import typing

EventStreamCloudEventConnectionCreatedObject3OptionsIdpinitiatedClientProtocolEnum = typing.Union[
    typing.Literal["oidc", "samlp", "wsfed"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_cloud_event_group_member_added_object_group_0 import (
    EventStreamCloudEventGroupMemberAddedObjectGroup0,
)
from .event_stream_cloud_event_group_member_added_object_group_1 import (
    EventStreamCloudEventGroupMemberAddedObjectGroup1,
)
from .event_stream_cloud_event_group_member_added_object_group_2 import (
    EventStreamCloudEventGroupMemberAddedObjectGroup2,
)

EventStreamCloudEventGroupMemberAddedObjectGroup = typing.Union[
    EventStreamCloudEventGroupMemberAddedObjectGroup0,
    EventStreamCloudEventGroupMemberAddedObjectGroup1,
    EventStreamCloudEventGroupMemberAddedObjectGroup2,
]
